/*   Elantech Passthrough Support
 *   Copyright (c) 2008 Elantech Inc Tom Lin<tom_lin@elantech.com.tw>
 *   
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Trademarks are the property of their respective owners.
 *
 */
#include "elaevent.h"
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include "elaproto.h"
#define ELANTECH_PRIVATE
#include "elantech.h"
#include <xf86.h>


#define SYSCALL(call) while (((call) == -1) && (errno == EINTR))

#define LONG_BITS (sizeof(long) * 8)
#define NBITS(x) (((x) + LONG_BITS - 1) / LONG_BITS)
#define OFF(x)   ((x) % LONG_BITS)
#define LONG(x)  ((x) / LONG_BITS)
#define TEST_BIT(bit, array) (array[LONG(bit)] & (1 << OFF(bit)))

/*****************************************************************************
 *      Function Definitions
 ****************************************************************************/

static void
EventDeviceOnHook(LocalDevicePtr local, ElantechSHM *para)
{
    if (para->grab_event_device) {
        /* Try to grab the event device so that data don't leak to /dev/input/mice */
        int ret;
        SYSCALL(ret = ioctl(local->fd, EVIOCGRAB, (pointer)1));
        if (ret < 0) {
            xf86Msg(X_WARNING, "%s can't grab event device, errno=%d\n",
                    local->name, errno);
        }
    }
}

static void
EventDeviceOffHook(LocalDevicePtr local)
{

}

static Bool
event_query_is_touchpad(int fd)
{
    int ret;
    unsigned long evbits[NBITS(KEY_MAX)];

    /* Check for ABS_X, ABS_Y, ABS_PRESSURE and BTN_TOOL_FINGER */

    SYSCALL(ret = ioctl(fd, EVIOCGBIT(0, EV_MAX), evbits));
    if (ret < 0)
        return FALSE;
    if (!TEST_BIT(EV_SYN, evbits) ||
        !TEST_BIT(EV_ABS, evbits) ||
        !TEST_BIT(EV_KEY, evbits))
        return FALSE;

    SYSCALL(ret = ioctl(fd, EVIOCGBIT(EV_ABS, KEY_MAX), evbits));
    if (ret < 0)
        return FALSE;
    if (!TEST_BIT(ABS_X, evbits) ||
        !TEST_BIT(ABS_Y, evbits) ||
        !TEST_BIT(ABS_PRESSURE, evbits))
        return FALSE;

    SYSCALL(ret = ioctl(fd, EVIOCGBIT(EV_KEY, KEY_MAX), evbits));
    if (ret < 0)
        return FALSE;
    if (!TEST_BIT(BTN_TOOL_FINGER, evbits))
        return FALSE;
    if (TEST_BIT(BTN_TOOL_PEN, evbits))
        return FALSE;                       /* Don't match wacom tablets */

    return TRUE;
}


/* Query device for axis ranges */
#if 1
static void
event_query_axis_ranges(LocalDevicePtr local)
{
    ElantechPrivate *priv = (ElantechPrivate *)local->private;
    struct input_absinfo abs;
  /*unsigned long absbits[NBITS(ABS_MAX)];
    unsigned long keybits[NBITS(KEY_MAX)];
    char buf[256];*/
    int rc;

    SYSCALL(rc = ioctl(local->fd, EVIOCGABS(ABS_X), &abs));
    if (rc >= 0)
    {
	xf86Msg(X_INFO, "%s: x-axis range %d - %d\n", local->name,
		abs.minimum, abs.maximum);
	priv->minx = abs.minimum;
	priv->maxx = abs.maximum;
    } else
	xf86Msg(X_ERROR, "%s: failed to query axis range (%s)\n", local->name,
		strerror(errno));

    SYSCALL(rc = ioctl(local->fd, EVIOCGABS(ABS_Y), &abs));
    if (rc >= 0)
    {
	xf86Msg(X_INFO, "%s: y-axis range %d - %d\n", local->name,
		abs.minimum, abs.maximum);
	priv->miny = abs.minimum;
	priv->maxy = abs.maximum;
    } else
	xf86Msg(X_ERROR, "%s: failed to query axis range (%s)\n", local->name,
		strerror(errno));
#if 0
    priv->has_pressure = FALSE;
    SYSCALL(rc = ioctl(local->fd, EVIOCGBIT(EV_ABS, sizeof(absbits)), absbits));
    if (rc >= 0)
	priv->has_pressure = TEST_BIT(ABS_PRESSURE, absbits);
    else
	xf86Msg(X_ERROR, "%s: failed to query ABS bits (%s)\n", local->name,
		strerror(errno));

    if (priv->has_pressure)
    {
	SYSCALL(rc = ioctl(local->fd, EVIOCGABS(ABS_PRESSURE), &abs));
	if (rc >= 0)
	{
	    xf86Msg(X_INFO, "%s: pressure range %d - %d\n", local->name,
		    abs.minimum, abs.maximum);
	    priv->minp = abs.minimum;
	    priv->maxp = abs.maximum;
	}
    } else
	xf86Msg(X_INFO,
		"%s: device does not report pressure, will use touch data.\n",
		local->name);

    SYSCALL(rc = ioctl(local->fd, EVIOCGABS(ABS_TOOL_WIDTH), &abs));
    if (rc >= 0)
    {
	xf86Msg(X_INFO, "%s: finger width range %d - %d\n", local->name,
		abs.minimum, abs.maximum);
	priv->minw = abs.minimum;
	priv->maxw = abs.maximum;
    }

    SYSCALL(rc = ioctl(local->fd, EVIOCGBIT(EV_KEY, sizeof(keybits)), keybits));
    if (rc >= 0)
    {
	buf[0] = 0;
	if ((priv->has_left = TEST_BIT(BTN_LEFT, keybits)))
	   strcat(buf, " left");
	if ((priv->has_right = TEST_BIT(BTN_RIGHT, keybits)))
	   strcat(buf, " right");
	if ((priv->has_middle = TEST_BIT(BTN_MIDDLE, keybits)))
	   strcat(buf, " middle");
	if ((priv->has_double = TEST_BIT(BTN_TOOL_DOUBLETAP, keybits)))
	   strcat(buf, " double");
	if ((priv->has_triple = TEST_BIT(BTN_TOOL_TRIPLETAP, keybits)))
	   strcat(buf, " triple");
	xf86Msg(X_INFO, "%s: buttons:%s\n", local->name, buf);
    }
#endif
}
#endif

static Bool
EventQueryHardware(LocalDevicePtr local)
{
    if (event_query_is_touchpad(local->fd)) {
        xf86Msg(X_PROBED, "%s touchpad found\n", local->name);
        return TRUE;
    }

    return FALSE;
}

static Bool
ElantechReadEvent(struct CommData *comm, struct input_event *ev)
{
    int i, c;
    unsigned char *pBuf, u;

    for (i = 0; i < sizeof(struct input_event); i++) {
        if ((c = XisbRead(comm->buffer)) < 0)
            return FALSE;
        u = (unsigned char)c;
        pBuf = (unsigned char *)ev;
        pBuf[i] = u;
    }
    return TRUE;
}

static Bool
EventReadHwState(LocalDevicePtr local, 
                 struct ElantechprotocolOperations *proto_ops,
                 struct CommData *comm, struct ElantechHwState *hwRet)
{
    struct input_event ev;
    Bool v;
    struct ElantechHwState *hw = &(comm->hwState);

    while (ElantechReadEvent(comm, &ev)) {
        switch (ev.type) {
        case EV_SYN:
            switch (ev.code) {
            case SYN_REPORT:
                if (comm->oneFinger)
                    hw->numFingers = 1;
                else if (comm->twoFingers)
                    hw->numFingers = 2;
                else if (comm->threeFingers)
                    hw->numFingers = 3;
                else
                    hw->numFingers = 0;
                *hwRet = *hw;
               
                return TRUE;
            } 
        case EV_KEY:
            v = (ev.value ? TRUE : FALSE);
            switch (ev.code) {
            case BTN_LEFT:
                hw->left = v;
                break;
            case BTN_RIGHT:
                hw->right = v;
                break;
            case BTN_MIDDLE:
                hw->middle = v;
                break;
            case BTN_1:
                hw->MKY = v;
                break;
            case BTN_2:
                hw->VF = v;
                break;
            case BTN_TOOL_FINGER:
                comm->oneFinger = v;
                break;
            case BTN_TOOL_DOUBLETAP:
                comm->twoFingers = v;
                break;
            case BTN_TOOL_TRIPLETAP:
                comm->threeFingers = v;
                break;

            }
            break;
       case EV_ABS:
            switch(ev.code){
                case ABS_X:
                        hw->x = ev.value;
			break;
                case ABS_Y:
                        hw->y = ev.value;
                        break;
                case ABS_HAT0X:
                        hw->x2_1 = ev.value;
                        break;
                case ABS_HAT0Y:
                        hw->y2_1 = ev.value;
                        break;
                case ABS_HAT1X:
                        hw->x2_2 = ev.value;
                        break;
                case ABS_HAT1Y:
                        hw->y2_2 = ev.value;
                        break;
                case ABS_HAT2X:
                        hw->x3_1 = ev.value;
                        break;
                case ABS_HAT2Y:
                        hw->y3_1 = ev.value;
                        break;
                case ABS_PRESSURE:
                        hw->z = ev.value;
                        break;
		case ABS_TOOL_WIDTH:
                        hw->w = ev.value;
                        break;
                }
            break;
       
        }
    }
    return FALSE;
}

/**
 * Probe the open device for dimensions.
 */

static void
EventReadDevDimensions(LocalDevicePtr local)
{
    if (event_query_is_touchpad(local->fd))
	event_query_axis_ranges(local);
}


static void
elantech_detect(LocalDevicePtr local)
{
    char tpname[255] = "Unknow";
    ElantechPrivate *priv = local->private;
    priv->has_pressure = 0;
    priv->smart_pad = 0;
    priv->convergesture=0;
    ioctl(local->fd,EVIOCGNAME(sizeof(tpname)),tpname);
    xf86Msg(X_INFO, "The toucpad name is %s\n",tpname);
    
    if(strstr(tpname,"Elantech ETF020022") != NULL){
	priv->byte_mode = 4;
	priv->version=ETF020022;
        xf86Msg(X_INFO,"Elantech Toucpad %d byte mode priv->version=%d\n",priv->byte_mode,priv->version);
    } else if(strstr(tpname,"Elantech ETF020600") != NULL){
	priv->byte_mode = 4;
	priv->version= ETF020600;
        xf86Msg(X_INFO,"Elantech Toucpad %d byte mode priv->version=%d\n",priv->byte_mode,priv->version);
    } else if(strstr(tpname,"Elantech ETF020030") != NULL){
	priv->byte_mode = 6;
	priv->version=ETF020030;
	priv->smart_pad = 1;
        xf86Msg(X_INFO,"Elantech Toucpad %d byte mode priv->version=%d\n",priv->byte_mode,priv->version);
    } else if(strstr(tpname,"Elantech ETF0208") != NULL){
      	priv->byte_mode = 6;
	priv->version=ETF0208;
	priv->has_pressure = 1;
	priv->smart_pad = 1;
	priv->convergesture=1;
        xf86Msg(X_INFO,"Elantech Toucpad %d byte mode priv->version=%d\n",priv->byte_mode,priv->version);
    } else if(strstr(tpname,"Elantech ETF020B00") != NULL){
      	priv->byte_mode = 6;
	priv->version=ETF020B00;
	priv->smart_pad = 1;
	priv->has_pressure = 1;
 	xf86Msg(X_INFO,"Elantech Toucpad %d byte mode priv->version=%d\n",priv->byte_mode,priv->version);
    } else if(strstr(tpname,"Elantech ETF0402") != NULL){
	priv->byte_mode = 6;
	priv->version=ETF0402;
	priv->has_pressure = 1;
	priv->smart_pad = 1;
        xf86Msg(X_INFO,"Elantech Toucpad %d byte mode priv->version=%d\n",priv->byte_mode,priv->version);
    } else if(strstr(tpname,"Elantech ETF0401") != NULL){
      	priv->byte_mode = 6;
	priv->version=ETF0401;
	priv->has_pressure = 1;
        xf86Msg(X_INFO,"Elantech Toucpad %d byte mode priv->version=%d\n",priv->byte_mode,priv->version);
    } else if(strstr(tpname,"Elantech ETF0403") != NULL){
      	priv->byte_mode = 6;
	priv->version=ETF0403;
	priv->has_pressure = 1;
        xf86Msg(X_INFO,"Elantech Toucpad %d byte mode priv->version=%d\n",priv->byte_mode,priv->version);
    }

}

static Bool
EventAutoDevProbe(LocalDevicePtr local)
{
    /* We are trying to find the right eventX device or fall back to
       the psaux protocol and the given device from XF86Config */
    int i,j;
    Bool have_evdev = FALSE;
    int noent_cnt = 0;
    const int max_skip = 10;
    
    /*ElantechPrivate *priv = local->private;*/

    for(j=0;j<30;j++){
		for (i = 0; ; i++) {
        		char fname[64];
        		int fd = -1;
        		Bool is_touchpad;

        		sprintf(fname, "%s/%s%d", DEV_INPUT_EVENT, EVENT_DEV_NAME, i);
        		SYSCALL(fd = open(fname, O_RDONLY));
        		if (fd < 0) {
            			if (errno == ENOENT) {
                			if (++noent_cnt >= max_skip)
                    				break;
                			else
                    				continue;
            			} else {
                			continue;
            			}
        		}
        		noent_cnt = 0;
        		have_evdev = TRUE;
        		is_touchpad = event_query_is_touchpad(fd);
        		if (is_touchpad) {
				local->fd = fd;
            			elantech_detect(local);            			
            			xf86Msg(X_PROBED, "%s auto-dev sets device to %s\n",
                    		local->name, fname);
            			xf86ReplaceStrOption(local->options, "Device", fname);
            			SYSCALL(close(fd));
            			return TRUE;
        		}
        		SYSCALL(close(fd));
    		}
		usleep(1000);
	}

    ErrorF("%s no Elatench event device found (checked %d nodes)\n",
           local->name, i + 1);
    if (i <= max_skip)
        ErrorF("%s The /dev/input/event* device nodes seem to be missing\n",
               local->name);
    if (i > max_skip && !have_evdev)
        ErrorF("%s The evdev kernel module seems to be missing\n", local->name);
    return FALSE;
}

struct ElantechProtocolOperations event_proto_operations = {
    EventDeviceOnHook,
    EventDeviceOffHook,
    EventQueryHardware,
    EventReadHwState,
    EventAutoDevProbe,
    EventReadDevDimensions
};
