/*   Elantech Passthrough Support
 *   Copyright (c) 2008 Elantech Inc Tom Lin<tom_lin@elantech.com.tw>
 *   Copyring  (c) 2009 ELAN 
 *   
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Trademarks are the property of their respective owners.
 *
 */
 
#ifndef _ELANTECH_H_
#define _ELANTECH_H_
/******************************************************************************
 *              Public definitions.
 *                      Used by driver and the shared memory configurator
 *****************************************************************************/
 typedef enum {
    RT_TAP = 0,                             /* Right top corner */
    RB_TAP,                                 /* Right bottom corner */
    LT_TAP,                                 /* Left top corner */
    LB_TAP,                                 /* Left bottom corner */
    F1_TAP,                                 /* Non-corner tap, one finger */
    F2_TAP,                                 /* Non-corner tap, two fingers */
    F3_TAP,                                 /* Non-corner tap, three fingers */
    MAX_TAP
} TapEvent;


typedef enum{
   ETF020022 = 0,
   ETF020600,/*4 byte mode fw_version*/
   ETF020030,      
   ETF020B00,
   ETF0208,
   ETF0402,
   ETF0401,
   ETF0403
} ElanFW;

typedef enum {
    BOTTOM_EDGE = 1, 
    RIGHT_EDGE   
} edge_type;

#define ELA_MAX_BUTTONS 12                  /* Max number of mouse buttons */
#define KEYMAP_SIZE 32
#define SHM_ELANTECH 29947

typedef struct _ElantechSHM
{
    int version;                            /* Driver version */
    int byte_mode;
    int has_pressure;
    int smart_pad;
    int convergesture;
    int new_struct;
    /* Current device state */
    int x, y;                               /* actual x, y coordinates */
    int z;                                  /* pressure value */
    int w;
    int x2_1,y2_1;
    int x2_2,y2_2;
    int x3_1,y3_1;
    int numFingers;                         /* number of fingers */
 
    int left, right, up, down;              /* left/right/up/down buttons */
    Bool MKY;
    Bool VF;
    Bool middle;
    /* Probed hardware properties */
   
    /* Parameter data */
    int left_edge, right_edge, top_edge, bottom_edge; /* edge coordinates absolute */
    int finger_low, finger_high, finger_press;	      /* finger detection values in Z-values */
    int tap_time;
    int tap_move;                           /* max. tapping time and movement in packets and coord. */
    int single_tap_timeout;                 /* timeout to recognize a single tap */
    int tap_time_2;                         /* max. tapping time for double taps */
    int click_time;                         /* The duration of a single click */
  
    int emulate_mid_button_time;            /* Max time between left and right button presses to
                                               emulate a middle button press. */
   
    int scroll_dist_vert;                   /* Scrolling distance in absolute coordinates */
    int scroll_dist_horiz;                  /* Scrolling distance in absolute coordinates */
    
    
    double min_speed, max_speed, accl;      /* movement parameters */
    double trackstick_speed;		    /* trackstick mode speed */
  
    Bool updown_button_scrolling;           /* Up/Down-Button scrolling or middle/double-click */
    Bool leftright_button_scrolling;        /* Left/right-button scrolling, or two lots of middle button */
    Bool updown_button_repeat;              /* If up/down button being used to scroll, auto-repeat?*/
    Bool leftright_button_repeat;           /* If left/right button being used to scroll, auto-repeat? */
    int scroll_button_repeat;               /* time, in milliseconds, between scroll events being
                                             * sent when holding down scroll buttons */
    int touchpad_off;                       /* Switches the touchpad off
                                             * 0 : Not off
                                             * 1 : Off
                                             * 2 : Only tapping and scrolling off
                                             */
    
    Bool locked_drags;                      /* Enable locked drags */
    int locked_drag_time;                   /* timeout for locked drags */
    int tap_action[MAX_TAP];                /* Button to report on tap events */
    int drag_lock;
    double coasting_speed;                  /* Coasting threshold scrolling speed */
    Bool grab_event_device;                 /* grab event device for exclusive use? */
    int  edge_v_scroll_area;
    int  edge_h_scroll_area;
    Bool can_edge_scroll_function;
    Bool all_can_move;
  
  /* new struck Parameter data */

    /*page_1 General*/


     /*page Button*/
	
    Bool button_checkBox;
    int button_left_comboBox;
    int button_middle_comboBox;
    int button_right_comboBox;
	
     /*page Tapping*/

    Bool tapping_checkBox;
    int two_FT_comboBox;
    int three_FT_comboBox;
    Bool disable_when_key_checkBox;
    int tapping_delay_comboBox;
    int	cursor_delay_comboBox;
    int gesture_delay_comboBox;
    int cover_delay_comboBox;
    int palmtrack_delay_comboBox;

    /*edge scroll*/
   
    Bool edge_v_scroll_checkBox;
    Bool edge_h_scroll_checkBox;

    int  edge_scroll_speed_slider;
    int  edge_scroll_area_slider;

     /*page Scrolling*/

    Bool vertical_scrolling_checkBox;
    Bool horizontal_scrolling_checkBox;
    int  scrolling_speed_slider;    
    Bool auto_scroll_checkBox;

     /*page Zoom*/

    int zoom_checkBox;
    int zoom_out_comboBox;
    int zoom_in_comboBox;
    int zoom_slider;

     /*page Rotation*/

    Bool rotation_checkBox;
    int clockwise_comboBox;
    int counterclosewise_comboBox;
    int rotation_slider;

     /*page Drag*/
     Bool drag_checkBox;
     Bool lift_release_radioButton;
     Bool tap_again_radioButton;
     Bool tap_release_radioButton;
     double drag_slider;

     /*page Swipe*/

    Bool swipe_page_checkBox;
	
    /*page Fast_keys*/

    Bool fast_key_checkBox;
    Bool my_home_file_checkBox;
    Bool switch_windows_checkBox;

    /*page Covergesture*/

    Bool covergesture_checkBox;
    int	covergesture_slider;
    int covergesture_comboBox;

    /*page PalmTracking*/
    Bool palmtracking_checkBox;
    int	palmtracking_slider;

    Bool detect_kbd_intr;
    Bool detect_tap_intr;
    Bool detect_gesture_intr;
    Bool detect_cover_intr;
    Bool detect_palmtrack_intr;

    Bool detect_ef023_tap_intr;

    Bool drag_timeout_release;
    
} ElantechSHM;

/*
 * Minimum and maximum values for scroll_button_repeat
 */
#define SBR_MIN 10
#define SBR_MAX 1000


#ifdef ELANTECH_PRIVATE

#include "elaproto.h"
/******************************************************************************
 *              Definitions
 *                                      structs, typedefs, #defines, enums
 *****************************************************************************/
#define ELANTECH_MOVE_HISTORY  5

typedef struct _ElantechMoveHist
{
    int x, y;
    int x2_1,y2_1;
    int x2_2,y2_2;
    int x3_1,y3_1;
    int millis;
    int millis_2;
    int millis_3;
} ElantechMoveHistRec;

enum FingerState {              /* Note! The order matters. Compared with < operator. */
    FS_UNTOUCHED,
    FS_TOUCHED,
    FS_PRESSED
};

enum MovingState {
    MS_FALSE,
    MS_TOUCHPAD_RELATIVE,
    MS_TRACKSTICK               /* trackstick is always relative */
};
enum MidButtonEmulation {
    MBE_OFF,                    /* No button pressed */
    MBE_LEFT,                   /* Left button pressed, waiting for right button or timeout */
    MBE_RIGHT,                  /* Right button pressed, waiting for left button or timeout */
    MBE_MID,                    /* Left and right buttons pressed, waiting for both buttons
                                   to be released */
    MBE_TIMEOUT                 /* Waiting for both buttons to be released. */
};

/* See docs/tapndrag.dia for a state machine diagram */
enum TapState {
    TS_START,                   /* No tap/drag in progress */
    TS_1,                       /* After first touch */
    TS_MOVE,                    /* Pointer movement enabled */
    TS_2A,                      /* After first release */
    TS_2B,                      /* After second/third/... release */
    TS_SINGLETAP,               /* After timeout after first release */
    TS_3,                       /* After second touch */
    TS_DRAG,                    /* Pointer drag enabled */
    TS_4,                       /* After release when "locked drags" enabled */
    TS_5                        /* After touch when "locked drags" enabled */
};

enum TapButtonState {
    TBS_BUTTON_UP,              /* "Virtual tap button" is up */
    TBS_BUTTON_DOWN,            /* "Virtual tap button" is down */
    TBS_BUTTON_DOWN_UP          /* Send button down event + set up state */
};

typedef struct _ElantechPrivateRec
{
     ElantechSHM                        elapara_default;                        /* Default parameter settings, read from
                                                                                   the X config file */
    ElantechSHM                         *elapara;                               /* Current parameter settings. Will point to
                                                                                   shared memory if shm_config is true */
    struct ElantechProtocolOperations*  proto_ops;
    
    struct ElantechHwState              hwState;
    
     Bool                                shm_config;                             /* True when shared memory area allocated */

    OsTimerPtr                          timer;                                  /* for up/down-button repeat, tap processing, etc */

    struct CommData                     comm;
    ElantechMoveHistRec                 move_hist[ELANTECH_MOVE_HISTORY];      /* movement history */
    int                                 hist_index;                             /* Last added entry in move_hist[] */
    int 				largest_valid_x;			/* Largest valid X coordinate seen so far */
     int                                 scroll_y;                               /* last y-scroll position */
    int                                 scroll_x;                               /* last x-scroll position */
  
    int                                 count_packet_finger;                    /* packet counter with finger on the touchpad */
    int                                 button_delay_millis;                    /* button delay for 3rd button emulation */
    Bool                                prev_up;                                /* Previous up button value, for double click emulation */
    enum                                FingerState finger_state;               /* previous finger state */
    
    enum TapState                       tap_state;                              /* State of tap processing */
    int                                 tap_max_fingers;                        /* Max number of fingers seen since entering start state */
    int                                 tap_button;                             /* Which button started the tap processing */
    enum TapButtonState                 tap_button_state;                       /* Current tap action */
    ElantechMoveHistRec                 touch_on;                               /* data when the touchpad is touched/released */
     enum MovingState                    moving_state;                           /* previous moving state */
  
    Bool                                vert_scroll_twofinger_on;               /* Keeps track of currently active scroll modes */
    Bool                                horiz_scroll_twofinger_on;              /* Keeps track of currently active scroll modes */
  
    int                                 trackstick_neutral_x;                   /* neutral x position for trackstick mode */
    int                                 trackstick_neutral_y;                   /* neutral y position for trackstick mode */
   
    int                                 scroll_packet_count;                    /* Scroll duration */
    double                              frac_x, frac_y;                         /* absolute -> relative fraction */
    enum MidButtonEmulation             mid_emu_state;                          /* emulated 3rd button */
    int                                 repeatButtons;                          /* buttons for repeat */
    int                                 nextRepeat;                             /* Time when to trigger next auto repeat event */
    int                                 lastButtons;                            /* last state of the buttons */
    int                                 palm;                                   /* Set to true when palm detected,reset to false when palm/finger contact disappears */
    int 				minx, maxx, miny, maxy;         /* min/max dimensions as detected */
    int 				minp, maxp, minw, maxw;		/* min/max pressure and finger width as detected */
    Bool				has_pressure;
    int                                 byte_mode;
    int					version;
    int					smart_pad;
    int					convergesture;
} ElantechPrivate;

#endif 
#endif